#!/usr/bin/env python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
"""
 direct_consumer.py

 This AMQP client reads messages from a message
 queue named "message_queue".
"""

import qpid
import sys
import os
from random import randint
from qpid.util import connect
from qpid.connection import Connection
from qpid.datatypes import Message, RangedSet, uuid4
from qpid.queue import Empty


#----- Initialization --------------------------------------

#  Set parameters for login

host="127.0.0.1"
port=5672
user="guest"
password="guest"

# If an alternate host or port has been specified, use that instead
# (this is used in our unit tests)
if len(sys.argv) > 1 :
  host=sys.argv[1]
if len(sys.argv) > 2 :
  port=int(sys.argv[2])


final = "That's all, folks!"   # In a message body, signals the last message
content = ""		       # Content of the last message read

#  Create a connection.
while content != final:
#if "true":

# print "initial while begun"
 try:
  socket = connect(host, port)
  connection = Connection (sock=socket, username=user, password=password)
  connection.start()
  session = connection.session(str(uuid4()))

#  print "connection/session created"

#----- Read from queue --------------------------------------------

# Now let's create a local client queue and tell it to read
# incoming messages.

# The consumer tag identifies the client-side queue.

  local_queue_name = "local_queue"
  queue = session.incoming(local_queue_name)

# Call message_subscribe() to tell the broker to deliver messages
# from the AMQP queue to this local client queue. The broker will
# start delivering messages as soon as credit is allocated using
# queue.start().

  session.message_subscribe(queue="message_queue", destination=local_queue_name)
  queue.start()
### alternative method of setting credit directly, instead of using defaults
#  session.message_set_flow_mode(local_queue_name, session.flow_mode.credit)
#  session.message_flow(local_queue_name, session.credit_unit.byte, 0xFFFFFFFFL)
#  session.message_flow(local_queue_name, session.credit_unit.message, 0xFFFFFFFFFL)
#  session.message_flow(local_queue_name, session.credit_unit.message, 500)

  message = None
#  print "starting inner while loop"

  if content != final:
	message = queue.get(timeout=2)
	content = message.body          
	dp = message.get("delivery_properties")
        session.message_accept(RangedSet(message.id))
        print message
        print "first msg:", content,
	try:
         print "red:", dp.redelivered
	except:
	 pass


  while content != final: 
	message = queue.get(timeout=2)
	content = message.body          
        session.message_accept(RangedSet(message.id))
#	print content

#----- Cleanup ------------------------------------------------

# Clean up before exiting so there are no open threads.
#
  print "close session"
  session.close(timeout=10)

 except qpid.session.SessionException:
  print "last msg:", message.body
#, "redeliv = ", message.isRedelivered()
  pass
 except KeyboardInterrupt:
  print "last msg:", message.body
  break
 except:
  exctype, value = sys.exc_info()[:2]
# print "got an exception - id is", message.id
#  print "got an exception, exctype is ", exctype, "value is ", value
  pass

